#ifndef _CGI_CLASS
#define _CGI_CLASS

#include "environ.h"
#include "fields.h"
#include "html.h"

class CGI
{
  public:
    virtual int runCGI ();       // these three functions can be
	 virtual int processfields(); // be re-written for each new script
	 virtual int processenv();    //  without creating new class

  private:
	 EnvVars ev;   // order of these classes is significant:
	 Fields flds;  //  EnvVars constructor must be called before Fields
	 HTML html;    //  Then HTML initiates output with http header


  public:
						// default constructor creates html file
	 CGI() :  flds(ev), html ("text/html"){ }

					 // allows for specification of alternate MIME type
	 CGI(char *content_type) :  flds(ev), html (content_type) { }

	  // Access functions to EnvVars Object
	 void display_env() const;
	 char const *getenv_var(int name) const;
	 EnvVars *getenvptr();


		 // Access functions to Fields Object
	 void display_fields() const ;
	 char *getfieldvalue(char *name);
	 void zerofldflags();
	 Fields *getfieldsptr();


};


	// display a formatted list of current environment variables
inline void CGI::display_env() const { ev.display(); }

       // display a formatted list of NAME=VALUE pairs
inline void CGI::display_fields() const { flds.display(); }


      // retrieve a value from a NAME=VALUE pair by specifiying
      // the field NAME; these are the NAMES used in the html form, eg:
      //    < INPUT type = "hidden" NAME = "field_1" VALUE = "default" >
inline char *CGI::getfieldvalue(char *name) { return flds.getvalue(name);}


    // The CGI Gateway protocol allows for multiple instances of the same
    // field NAME. When the Field object retrieves a VALUE for an instance
    // of a NAME, it sets the field's retrieved flag = 1, so that it will
	 // not retrieve the same VALUE when the next instance of that NAME
	 // is passed to getfieldvalue().  If you want to backtrack and get an
	 // earlier, already retrieved instance OF NAME, reset the retrieved flags
	 // to zero using zerofldflags().
inline void CGI::zerofldflags() { flds.reset(); }



	  // retrieve an environment variable specified by enumeration
	  // constants defined in the ENVIRONMENT_VALUES enum (see "environ.h")
	  // these constants are the CGI environment variable names
	  // preceded by and underscore, as in:
	  //      _QUERY_STRING,   _CONTENT_LENGTH,  etc.
inline char const *CGI::getenv_var(int name) const { return ev.env_var(name); }





	// this returns a pointer to the Field object created when initializing
	// the CGI object, thus providing access to the public members of
	// the Field object
inline Fields *CGI::getfieldsptr() { return &flds; }


	// same as the above, but for the EnvVars object
inline EnvVars *CGI::getenvptr() { return &ev; }

/*

	void processfields();  // to be re-written for each program
	void processenv();    // to be re-written for each program

	These two functions are where the work of each implementation shold take
	place.  They have access to all of the necessary members of each
	class.  However, the CGI class gives users access to the member
	classes through getfieldsptr() and getenvptr() as well as through
	the access functions specfic to each class.  A simple HTML form could
	quickly and easily be handled within the main() function itself,
	without reference to the process...() functions.

	If you simply want to see your environment variables, for instance,
	you could create the following main() function:
					void main(void)
					{
					 CGI cgi;
					 HTML html;

					 cgi.display_env();
					 
				  }


*/

#endif









