#include "fields.h"
#include "cgiform.h"
Fields *Fields::FirstInstance = 0;
int Fields::RetrieveFlagValue = 1;
char *Fields::FieldStrBuf = 0;
int __strnicmp(const char *str1, const char *str2, size_t len);

Fields::Fields(EnvVars& ev)
{
	fieldptrs = 0;
	fieldstring = 0;


		 // all subsequent instances = FirstInstance: uses Fields::operator=
       // unless FirstInstance has been destroyed: then convert() is used
	if(FirstInstance) { *this = *FirstInstance; 	return; 	}

	FirstInstance = this;
   
   if(FieldStrBuf) {
       convert();
       return;
   }



	if (ev.cgi_method() == EnvVars::_GET)
		 {
			  // preserve QUERY_STRING for display() since fieldstring gets
			  // chopped up into name/value pairs
		  char const *qs;
		  if ( (qs = ev.env_var(_QUERY_STRING)) )
			 {
			 size_t strL = strlen(qs);
			 fieldstring = new char [strL + 1];
			 strcpy(fieldstring, qs)[strL] = '\0';
			 setfields();
			 }
		 }
	  else if (ev.content_length && ev.cgi_method() == EnvVars::_POST)
	  {
		fieldstring = new char [ev.content_length + 1] ;
      if (!fieldstring) exit(0);

		int i = 0, x;
		  while (i<ev.content_length) {
				x = fgetc(stdin);
				if (x==EOF) break;
				fieldstring[i++] = (char)x;
		  }
		  fieldstring[i] = '\0';
        if (!FieldStrBuf) {
            FieldStrBuf = new char [ev.content_length + 1] ;
            if (!FieldStrBuf) exit(0);
            strcpy(FieldStrBuf, fieldstring);
            }
		  setfields();
	  }




}


Fields::Fields(Fields &other)
{

  size_t strL = strlen (other.fieldstring);
  fieldstring = new char [strL + 1];
  if (!fieldstring) exit(0);
  strcpy(fieldstring, other.fieldstring);


  if(!other.fieldptrs) return;

  FIELDPAIRS *otherfp = other.fieldptrs,  *prev=0;
  FIELDPAIRS *current_this = fieldptrs = new FIELDPAIRS;
  if (!current_this) return;

  while (otherfp)
  {

	  if(current_this)
	  {

			 current_this->name = otherfp->name;
			 current_this->value = otherfp->value;
			 current_this->retrieved = 0;
			 if(prev)
				 prev->next = current_this;
			 prev = current_this;
			 current_this->next = 0;
	  }
	  else break;
	  otherfp = otherfp->next;
	  current_this = new FIELDPAIRS;
  }




}


Fields &Fields::operator=(Fields &other)
{

  size_t strL = strlen (other.fieldstring);

  if (&other == this) return *this;

  fieldstring = new char [strL + 1];
  if (!fieldstring) exit(0);
  strcpy(fieldstring, other.fieldstring);


  if(!other.fieldptrs) return *this;

  FIELDPAIRS *otherfp = other.fieldptrs,  *prev=0;
  FIELDPAIRS *current_this = fieldptrs = new FIELDPAIRS;
  if (!current_this) return *this;

  while (otherfp)
  {

	  if(current_this)
	  {

			 current_this->name = otherfp->name;
			 current_this->value = otherfp->value;
			 current_this->retrieved = 0;
			 if(prev)
				 prev->next = current_this;
			 prev = current_this;
			 current_this->next = 0;
	  }
	  else break;
	  otherfp = otherfp->next;
	  current_this = new FIELDPAIRS;
  }


	return *this;

}

Fields::~Fields()
{
	FIELDPAIRS *temp;

	delete fieldstring;

	while (fieldptrs)
	{
	 temp = fieldptrs->next;
	 delete fieldptrs;
	 fieldptrs = temp;
	}

  if(this == FirstInstance)
          FirstInstance = 0;



}
void Fields::setfields()
{
	 char *p;
	 FIELDPAIRS *prev = 0, *fldptr = fieldptrs = new FIELDPAIRS;

	 if(!fieldstring) return;

		  p = strtok(fieldstring,"&");
		  while (p) {
			 fldptr->name = p;
			 fldptr->value = strchr(fldptr->name, '=');
			 if (fldptr->value) {
				 *fldptr->value  = '\0';
				  fldptr->value++;
			 }
			 fldptr->retrieved = 0;
			 unescape_url(fldptr->name);
			 plustospace(fldptr->name);
			 unescape_url(fldptr->value);
			 plustospace(fldptr->value);
			 prev = fldptr;
			 fldptr = new FIELDPAIRS;
			 if(fldptr)
				 prev->next = fldptr;
			  else break;
			p = strtok(NULL,"&");
		  }
		 prev->next = 0;
}

void Fields::reset()
{
	FIELDPAIRS *p = fieldptrs;

	while (p)
	{
	 p->retrieved = 0;
	 p = p->next;
	}

}

void Fields::reset(char *name)
{
	FIELDPAIRS *p = fieldptrs;
	size_t strL = strlen(name);
	while (p)
	{
	if( __strnicmp(p->name, name, strL) == 0)
		  p->retrieved = 0;
	 p = p->next;
	}

}


 /*(p->retrieved + ) <= 0*/
char *Fields::getvalue(char *name) const
{
	FIELDPAIRS *p = fieldptrs;


	 while(p)
	 {
		if (!p->retrieved && __strnicmp(p->name, name, strlen(name)) == 0)
			{
			 p->retrieved = RetrieveFlagValue;   // 1;
			 return (p->value);
			}

		p = p->next;
	 }

	 return 0;
}

void Fields::convert()
{
    size_t strL = strlen(FieldStrBuf);
 	 fieldstring = new char [strL + 1] ;
    if (!fieldstring) exit(0);

    strcpy(fieldstring, FieldStrBuf);
    setfields();


}
void Fields::display()  const
{
	 FIELDPAIRS *p = fieldptrs;


	 while (p)
	 {
		cout << p->name << " = " << p->value << "<br>\n";
		p = p->next;
	 }
}
// Helper functions

char x2c(char *what) {
	 int digit;

	 digit = (what[0] >= 'A' ? ((what[0] & 0xdf) - 'A')+10 : (what[0] - '0'));
	 digit *= 16;
	 digit += (what[1] >= 'A' ? ((what[1] & 0xdf) - 'A')+10 : (what[1] - '0'));
	 return((char)digit);
}


void unescape_url(char *url) {
	 register int x,y;

	 for(x=0,y=0;url[y];++x,++y) {
		  if((url[x] = url[y]) == '%') {
				url[x] = x2c(&url[y+1]);
				y+=2;
		  }
	 }
	 url[x] = '\0';
}

void plustospace(char *str) {
	 register int x;

	 for(x=0;str[x];x++) if(str[x] == '+') str[x] = ' ';
}



